@@ -1,5 +1,5 @@
 
-import re
+import re # pylint: disable=unused-import
 
 from repoman.modules.linechecks.base import LineCheck
 
@@ -19,7 +19,7 @@ class DeprecatedHasq(LineCheck):
 
 
 class PreserveOldLib(LineCheck):
-	"""Check for calls to the deprecated preserve_old_lib function."""
+	"""Check for calls to the preserve_old_lib function reserved for system packages."""
 	repoman_check_name = 'ebuild.minorsyn'
 	re = re.compile(r'.*preserve_old_lib')
 	error = 'PRESERVE_OLD_LIB'
