@@ -1,4 +1,4 @@
-# Copyright 2015-2016 Gentoo Foundation
+# Copyright 2015-2021 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
 doc = """Phases plug-in module for repoman LineChecks.
@@ -29,7 +29,12 @@ module_spec = {
 			'class': "SrcUnpackPatches",
 			'description': doc,
 		},
+		'pmsvariablerefphasescope-check': {
+			'name': "pmsvariablerefphasescope",
+			'sourcefile': "phase",
+			'class': "PMSVariableReference",
+			'description': doc,
+		},
 	},
 	'version': 1,
 }
-
