@@ -1,21 +1,88 @@
-# Copyright 2011-2018 Gentoo Foundation
+# Copyright 2011-2021 Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
 
+import collections
 import subprocess
 import sys
 import time
+import types
 
 from repoman._portage import portage
 from portage import os
-from portage import _unicode_decode
 from portage.process import find_binary
 from portage.tests.resolver.ResolverPlayground import ResolverPlayground
 from portage.util import ensure_dirs
+from portage.util.futures import asyncio
+from portage.util.futures._asyncio.streams import _reader
+from portage.util._async.AsyncFunction import AsyncFunction
+
+# pylint: disable=ungrouped-imports
 from repoman import REPOMAN_BASE_PATH
 from repoman.copyrights import update_copyright_year
+from repoman.main import _repoman_init, _repoman_scan, _handle_result
 from repoman.tests import TestCase
 
 
+class RepomanRun(types.SimpleNamespace):
+	async def run(self):
+		self.expected = getattr(self, "expected", None) or {"returncode": 0}
+		if self.debug:
+			fd_pipes = {}
+			pr = None
+			pw = None
+		else:
+			pr, pw = os.pipe()
+			fd_pipes = {1: pw, 2: pw}
+			pr = open(pr, "rb", 0)
+
+		proc = AsyncFunction(
+			scheduler=asyncio.get_event_loop(),
+			target=self._subprocess,
+			args=(self.args, self.cwd, self.env, self.expected, self.debug),
+			fd_pipes=fd_pipes,
+		)
+
+		proc.start()
+		if pw is not None:
+			os.close(pw)
+
+		await proc.async_wait()
+
+		if pr is None:
+			stdio = None
+		else:
+			stdio = await _reader(pr)
+
+		self.result = {
+			"stdio": stdio,
+			"result": proc.result,
+		}
+
+	@staticmethod
+	def _subprocess(args, cwd, env, expected, debug):
+		os.chdir(cwd)
+		os.environ.update(env)
+		repoman_vars = _repoman_init(["repoman"] + args)
+		if repoman_vars.exitcode is not None:
+			return repoman_vars.exitcode
+		result = _repoman_scan(*repoman_vars)
+		returncode = _handle_result(*repoman_vars, result)
+		qawarnings = repoman_vars.vcs_settings.qatracker.qawarnings
+		warns = collections.defaultdict(list)
+		fails = collections.defaultdict(list)
+		for qacat, issues in repoman_vars.vcs_settings.qatracker.fails.items():
+			if qacat in qawarnings:
+				warns[qacat].extend(issues)
+			else:
+				fails[qacat].extend(issues)
+		result = {"returncode": returncode}
+		if fails:
+			result["fails"] = fails
+		if warns:
+			result["warns"] = warns
+		return result
+
+
 class SimpleRepomanTestCase(TestCase):
 
 	def testCopyrightUpdate(self):
@@ -70,11 +137,17 @@ class SimpleRepomanTestCase(TestCase):
 			self.assertFalse(True, skip_reason)
 			return
 
-		copyright_header = """# Copyright 1999-%s Gentoo Foundation
+		copyright_header = """# Copyright 1999-%s Gentoo Authors
 # Distributed under the terms of the GNU General Public License v2
-# $Header: $
+
 """ % time.gmtime().tm_year
 
+		pkg_preinst_references_forbidden_var = """
+pkg_preinst() {
+	echo "This ${A} reference is not allowed. Neither is this $BROOT reference."
+}
+"""
+
 		repo_configs = {
 			"test_repo": {
 				"layout.conf":
@@ -128,13 +201,14 @@ class SimpleRepomanTestCase(TestCase):
 			"dev-libs/C-0": {
 				"COPYRIGHT_HEADER" : copyright_header,
 				"DESCRIPTION" : "Desc goes here",
-				"EAPI" : "4",
+				"EAPI" : "7",
 				"HOMEPAGE" : "https://example.com",
 				"IUSE" : "flag",
 				# must be unstable, since dev-libs/A[flag] is stable masked
 				"KEYWORDS": "~x86",
 				"LICENSE": "GPL-2",
 				"RDEPEND": "flag? ( dev-libs/A[flag] )",
+				"MISC_CONTENT": pkg_preinst_references_forbidden_var,
 			},
 		}
 		licenses = ["GPL-2"]
@@ -167,6 +241,39 @@ class SimpleRepomanTestCase(TestCase):
 
 		playground = ResolverPlayground(ebuilds=ebuilds,
 			profile=profile, repo_configs=repo_configs, debug=debug)
+
+		loop = asyncio._wrap_loop()
+		loop.run_until_complete(
+			asyncio.ensure_future(
+				self._async_test_simple(
+					playground,
+					metadata_xml_files,
+					profiles,
+					profile,
+					licenses,
+					arch_list,
+					use_desc,
+					metadata_xsd,
+					copyright_header,
+					debug,
+				),
+				loop=loop,
+			)
+		)
+
+	async def _async_test_simple(
+		self,
+		playground,
+		metadata_xml_files,
+		profiles,
+		profile,
+		licenses,
+		arch_list,
+		use_desc,
+		metadata_xsd,
+		copyright_header,
+		debug,
+	):
 		settings = playground.settings
 		eprefix = settings["EPREFIX"]
 		eroot = settings["EROOT"]
@@ -193,25 +300,34 @@ class SimpleRepomanTestCase(TestCase):
 
 		committer_name = "Gentoo Dev"
 		committer_email = "gentoo-dev@gentoo.org"
+		expected_warnings = {
+			"returncode": 0,
+			"warns": {
+				"variable.phase": [
+					"dev-libs/C/C-0.ebuild: line 15: phase pkg_preinst: EAPI 7: variable A: Forbidden reference to variable specified by PMS",
+					"dev-libs/C/C-0.ebuild: line 15: phase pkg_preinst: EAPI 7: variable BROOT: Forbidden reference to variable specified by PMS",
+				]
+			},
+		}
 
 		git_test = (
-			("", repoman_cmd + ("manifest",)),
+			("", RepomanRun(args=["manifest"])),
 			("", git_cmd + ("config", "--global", "user.name", committer_name,)),
 			("", git_cmd + ("config", "--global", "user.email", committer_email,)),
 			("", git_cmd + ("init-db",)),
 			("", git_cmd + ("add", ".")),
 			("", git_cmd + ("commit", "-a", "-m", "add whole repo")),
-			("", repoman_cmd + ("full", "-d")),
-			("", repoman_cmd + ("full", "--include-profiles", "default/linux/x86/test_profile")),
+			("", RepomanRun(args=["full", "-d"], expected=expected_warnings)),
+			("", RepomanRun(args=["full", "--include-profiles", "default/linux/x86/test_profile"], expected=expected_warnings)),
 			("", cp_cmd + (test_ebuild, test_ebuild[:-8] + "2.ebuild")),
 			("", git_cmd + ("add", test_ebuild[:-8] + "2.ebuild")),
-			("", repoman_cmd + ("commit", "-m", "cat/pkg: bump to version 2")),
+			("", RepomanRun(args=["commit", "-m", "cat/pkg: bump to version 2"], expected=expected_warnings)),
 			("", cp_cmd + (test_ebuild, test_ebuild[:-8] + "3.ebuild")),
 			("", git_cmd + ("add", test_ebuild[:-8] + "3.ebuild")),
-			("dev-libs", repoman_cmd + ("commit", "-m", "cat/pkg: bump to version 3")),
+			("dev-libs", RepomanRun(args=["commit", "-m", "cat/pkg: bump to version 3"], expected=expected_warnings)),
 			("", cp_cmd + (test_ebuild, test_ebuild[:-8] + "4.ebuild")),
 			("", git_cmd + ("add", test_ebuild[:-8] + "4.ebuild")),
-			("dev-libs/A", repoman_cmd + ("commit", "-m", "cat/pkg: bump to version 4")),
+			("dev-libs/A", RepomanRun(args=["commit", "-m", "cat/pkg: bump to version 4"])),
 		)
 
 		env = {
@@ -280,41 +396,66 @@ class SimpleRepomanTestCase(TestCase):
 				# triggered by python -Wd will be visible.
 				stdout = subprocess.PIPE
 
-			for cwd in ("", "dev-libs", "dev-libs/A", "dev-libs/B"):
+			for cwd in ("", "dev-libs", "dev-libs/A", "dev-libs/B", "dev-libs/C"):
 				abs_cwd = os.path.join(test_repo_symlink, cwd)
-				proc = subprocess.Popen(repoman_cmd + ("full",),
-					cwd=abs_cwd, env=env, stdout=stdout)
+
+				proc = await asyncio.create_subprocess_exec(
+					*(repoman_cmd + ("full",)),
+					env=env,
+					stderr=None,
+					stdout=stdout,
+					cwd=abs_cwd
+				)
 
 				if debug:
-					proc.wait()
+					await proc.wait()
 				else:
-					output = proc.stdout.readlines()
-					proc.wait()
-					proc.stdout.close()
+					output, _err = await proc.communicate()
+					await proc.wait()
 					if proc.returncode != os.EX_OK:
-						for line in output:
-							sys.stderr.write(_unicode_decode(line))
+						portage.writemsg(output)
 
-				self.assertEqual(os.EX_OK, proc.returncode,
-					"repoman failed in %s" % (cwd,))
+				self.assertEqual(
+					os.EX_OK, proc.returncode, "repoman failed in %s" % (cwd,)
+				)
 
 			if git_binary is not None:
 				for cwd, cmd in git_test:
 					abs_cwd = os.path.join(test_repo_symlink, cwd)
-					proc = subprocess.Popen(cmd,
-						cwd=abs_cwd, env=env, stdout=stdout)
+					if isinstance(cmd, RepomanRun):
+						cmd.cwd = abs_cwd
+						cmd.env = env
+						cmd.debug = debug
+						await cmd.run()
+						if cmd.result["result"] != cmd.expected and cmd.result.get("stdio"):
+							portage.writemsg(cmd.result["stdio"])
+						try:
+							self.assertEqual(cmd.result["result"], cmd.expected)
+						except Exception:
+							print(cmd.result["result"], file=sys.stderr, flush=True)
+							raise
+						continue
+
+					proc = await asyncio.create_subprocess_exec(
+						*cmd, env=env, stderr=None, stdout=stdout, cwd=abs_cwd
+					)
 
 					if debug:
-						proc.wait()
+						await proc.wait()
 					else:
-						output = proc.stdout.readlines()
-						proc.wait()
-						proc.stdout.close()
+						output, _err = await proc.communicate()
+						await proc.wait()
 						if proc.returncode != os.EX_OK:
-							for line in output:
-								sys.stderr.write(_unicode_decode(line))
+							portage.writemsg(output)
 
-					self.assertEqual(os.EX_OK, proc.returncode,
-						"%s failed in %s" % (cmd, cwd,))
+					self.assertEqual(
+						os.EX_OK,
+						proc.returncode,
+						"%s failed in %s"
+						% (
+							cmd,
+							cwd,
+						),
+					)
 		finally:
 			playground.cleanup()
